@@ -34,6 +34,7 @@
 #include <linux/delay.h>
 #include <linux/timer.h>
 #include <linux/interrupt.h>
+#include <linux/mutex.h>
 #include <asm/unaligned.h>
 #include <linux/miscdevice.h>
 #include <mach/gpio.h>
@@ -110,6 +111,7 @@ enum {
 	REG_CAPACITY,
 	REG_SERIAL_NUMBER,
 	REG_CHARGE_NOW,
+	REG_ENERGY,
 	REG_POWER,
 	REG_CYCLE,
 
@@ -141,6 +143,7 @@ static struct bq27541_device_data {
 	[REG_CAPACITY]				= BQ27541_DATA(CAPACITY, 0x2c, 0, 100),
 
 	[REG_CHARGE_NOW]			= BQ27541_DATA(CHARGE_NOW, 0x10, 0, 65535),
+	[REG_ENERGY]				= BQ27541_DATA(ENERGY_NOW, 0x22, 0, 65535),
 	[REG_POWER]				= BQ27541_DATA(POWER_AVG, 0x24, 0, 65535),
 	[REG_CYCLE]				= BQ27541_DATA(CYCLE_COUNT, 0x2a, 0, 65535),
 
@@ -155,6 +158,7 @@ static enum power_supply_property bq27541_properties[] = {
 	POWER_SUPPLY_PROP_CAPACITY,
 	POWER_SUPPLY_PROP_TEMP,
 	POWER_SUPPLY_PROP_CHARGE_NOW,
+	POWER_SUPPLY_PROP_ENERGY_NOW,
 	POWER_SUPPLY_PROP_CYCLE_COUNT
 };
 
@@ -265,7 +269,7 @@ static struct bq27541_device_info {
 	unsigned int old_temperature;
 	unsigned int temp_err;
 	unsigned int prj_id;
-	spinlock_t lock;
+	struct mutex lock;
 } *bq27541_device;
 
 static int bq27541_read_i2c(u8 reg, int *rt_value, int b_single)
@@ -504,6 +508,11 @@ static int bq27541_get_psp(int reg_offset, enum power_supply_property psp,
 		val->intval = rt_value * 1000;
 		BAT_NOTICE("charge_now = %u uA\n", val->intval);
 	}
+	if (psp == POWER_SUPPLY_PROP_ENERGY_NOW) {
+		/* energy is reported in 10 mWh */
+		val->intval = rt_value * 10000;
+		BAT_NOTICE("energy_now = %u uWh\n", val->intval);
+	}
 	if (psp == POWER_SUPPLY_PROP_CYCLE_COUNT) {
 		val->intval = rt_value;
 		BAT_NOTICE("cycle count = %u\n", val->intval);
@@ -689,6 +698,9 @@ static int bq27541_get_property(struct power_supply *psy,
 	union power_supply_propval *val)
 {
 	u8 count;
+
+	mutex_lock(&bq27541_device->lock);
+
 	switch (psp) {
 		case POWER_SUPPLY_PROP_PRESENT:
 		case POWER_SUPPLY_PROP_HEALTH:
@@ -710,6 +722,7 @@ static int bq27541_get_property(struct power_supply *psy,
 		case POWER_SUPPLY_PROP_TEMP:
 		case POWER_SUPPLY_PROP_SERIAL_NUMBER:
 		case POWER_SUPPLY_PROP_CHARGE_NOW:
+		case POWER_SUPPLY_PROP_ENERGY_NOW:
 		case POWER_SUPPLY_PROP_CYCLE_COUNT:
 			for (count = 0; count < REG_MAX; count++) {
 				if (psp == bq27541_data[count].psp)
@@ -717,19 +730,21 @@ static int bq27541_get_property(struct power_supply *psy,
 			}
 
 			if (bq27541_get_psp(count, psp, val))
-				return -EINVAL;
+				goto error;
 			break;
 
 		default:
 			dev_err(&bq27541_device->client->dev,
 				"%s: INVALID property psp=%u\n", __func__,psp);
-			return -EINVAL;
+			goto error;
 	}
 
+	mutex_unlock(&bq27541_device->lock);
 	return 0;
 
 error:
 
+	mutex_unlock(&bq27541_device->lock);
 	return -EINVAL;
 }
 
@@ -757,6 +772,8 @@ static int bq27541_probe(struct i2c_client *client,
 	bq27541_device->shutdown_disable = 1;
 	bq27541_device->cap_zero_count = 0;
 
+	mutex_init(&bq27541_device->lock);
+
 	for (i = 0; i < ARRAY_SIZE(bq27541_supply); i++) {
 		ret = power_supply_register(&client->dev, &bq27541_supply[i]);
 		if (ret) {
@@ -775,7 +792,6 @@ static int bq27541_probe(struct i2c_client *client,
 	INIT_DELAYED_WORK(&bq27541_device->shutdown_en_work, shutdown_enable_set);
 	cancel_delayed_work(&bq27541_device->status_poll_work);
 
-	spin_lock_init(&bq27541_device->lock);
 	wake_lock_init(&bq27541_device->low_battery_wake_lock, WAKE_LOCK_SUSPEND, "low_battery_detection");
 	wake_lock_init(&bq27541_device->cable_wake_lock, WAKE_LOCK_SUSPEND, "cable_state_changed");
 
